@@ -1,6 +1,8 @@
 #undef PACKAGE
 #undef VERSION
 
+#undef PREFIX
+
 #undef DRIVER_OSS
 #undef DRIVER_ALSA
 #undef DRIVER_ALSA_050
@@ -10,9 +12,7 @@
 #undef NO_AUDIOFILE
 #undef NO_GASP
 #undef NO_ASM
+#undef NO_GDK_PIXBUF
 
 #undef ENABLE_NLS
 #undef HAVE_CATGETS
-#undef HAVE_GETTEXT
-#undef HAVE_LC_MESSAGES
-#undef HAVE_STPCPY
