@@ -23,6 +23,9 @@
 
 #include <string.h>
 
+#include <errno.h>
+#include <dirent.h>
+
 #include "i18n.h"
 #include <gdk/gdktypes.h>
 #include "file-operations.h"
@@ -224,3 +227,31 @@ fileops_open_dialog (void *dummy,
 	gtk_widget_show(fileops_dialogs[n]);
     }
 }
+
+/* simple, non-recursive file eraser... make it better! :-) */
+void
+fileops_tmpclean (void)
+{
+    DIR *dire;
+    struct dirent *entry;
+    static char tname[1024], fname[1024];
+
+    strcpy (tname, prefs_get_prefsdir());
+    strcat (tname, "/tmp/");
+
+    if(!(dire = opendir(tname))) {
+	return;
+    }
+
+    while((entry = readdir(dire))) {
+	if (strcmp(entry->d_name, ".") && strcmp(entry->d_name, "..")) {
+	    strcpy(fname, tname);
+	    strcat(fname, entry->d_name);
+	    unlink(fname);
+        }
+    }
+
+    closedir(dire);
+}
+
+
