@@ -46,18 +46,16 @@ gui_prefs gui_settings = {
     16,
     8,
 
-
     0,
     0,
-    1,
 
     1,
     0,
 
     1,
     0,
-    0,
 
+    0,
     1,
 
     50,
@@ -66,6 +64,11 @@ gui_prefs gui_settings = {
 
     1,
 
+    -666,
+    0,
+    0,
+    0,
+
     "~/",
     "~/",
     "~/",
@@ -149,15 +152,6 @@ gui_settings_asyncedit_toggled (GtkWidget *widget)
     gui_settings.asynchronous_editing = GTK_TOGGLE_BUTTON(widget)->active;
 }
 
-static void
-gui_settings_channel_numbering_toggled (GtkWidget *widget)
-{
-    int o = gui_settings.channel_numbering;
-    if(o != (gui_settings.channel_numbering = GTK_TOGGLE_BUTTON(widget)->active)) {
-	gtk_widget_queue_resize(GTK_WIDGET(tracker));
-    }
-}
-
 static void
 gui_settings_tempo_bpm_update_toggled (GtkWidget *widget)
 {
@@ -177,9 +171,12 @@ gui_settings_auto_switch_toggled (GtkWidget *widget)
 }
 
 static void
-gui_settings_use_aa_canvas_toggled (GtkWidget *widget)
+gui_settings_save_geometry_toggled (GtkWidget *widget)
 {
-    gui_settings.gui_use_aa_canvas = GTK_TOGGLE_BUTTON(widget)->active;
+    int o = gui_settings.save_geometry;
+    if(o != (gui_settings.save_geometry = GTK_TOGGLE_BUTTON(widget)->active)) {
+       gtk_widget_queue_resize(GTK_WIDGET(tracker));
+    }
 }
 
 static void
@@ -364,48 +361,40 @@ gui_settings_dialog (void)
     gtk_signal_connect(GTK_OBJECT(thing), "toggled",
 		       GTK_SIGNAL_FUNC(gui_settings_upcase_toggled), NULL);
 
-    thing = gtk_check_button_new_with_label(_("Advance cursor horizontally in effect columns"));
+    thing = gtk_check_button_new_with_label(_("Horizontal effect column editing"));
     gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(thing), gui_settings.advance_cursor_in_fx_columns);
     gtk_box_pack_start(GTK_BOX(vbox1), thing, FALSE, TRUE, 0);
     gtk_widget_show(thing);
     gtk_signal_connect(GTK_OBJECT(thing), "toggled",
 		       GTK_SIGNAL_FUNC(gui_settings_advance_cursor_toggled), NULL);
 
-    thing = gtk_check_button_new_with_label(_("Asynchronous Editing"));
+    thing = gtk_check_button_new_with_label(_("Asynchronous (IT-style) pattern editing"));
     gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(thing), gui_settings.asynchronous_editing);
     gtk_box_pack_start(GTK_BOX(vbox1), thing, FALSE, TRUE, 0);
     gtk_widget_show(thing);
     gtk_signal_connect(GTK_OBJECT(thing), "toggled",
 		       GTK_SIGNAL_FUNC(gui_settings_asyncedit_toggled), NULL);
 
-    thing = gtk_check_button_new_with_label(_("Channel numbering"));
-    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(thing), gui_settings.channel_numbering);
-    gtk_box_pack_start(GTK_BOX(vbox1), thing, FALSE, TRUE, 0);
-    gtk_widget_show(thing);
-    gtk_signal_connect(GTK_OBJECT(thing), "toggled",
-		       GTK_SIGNAL_FUNC(gui_settings_channel_numbering_toggled), NULL);
-
-    thing = gtk_check_button_new_with_label(_("Tempo and BPM update"));
+    thing = gtk_check_button_new_with_label(_("Fxx command updates Tempo/BPM sliders"));
     gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(thing), gui_settings.tempo_bpm_update);
     gtk_box_pack_start(GTK_BOX(vbox1), thing, FALSE, TRUE, 0);
     gtk_widget_show(thing);
     gtk_signal_connect(GTK_OBJECT(thing), "toggled",
 		       GTK_SIGNAL_FUNC(gui_settings_tempo_bpm_update_toggled), NULL);
 
-    thing = gtk_check_button_new_with_label(_("Auto switch"));
+    thing = gtk_check_button_new_with_label(_("Switch to tracker after loading/saving"));
     gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(thing), gui_settings.auto_switch);
     gtk_box_pack_start(GTK_BOX(vbox1), thing, FALSE, TRUE, 0);
     gtk_widget_show(thing);
     gtk_signal_connect(GTK_OBJECT(thing), "toggled",
 		       GTK_SIGNAL_FUNC(gui_settings_auto_switch_toggled), NULL);
 
-    thing = gtk_check_button_new_with_label(_("Use anti-aliased envelope editor"));
-    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(thing), gui_settings.gui_use_aa_canvas);
+    thing = gtk_check_button_new_with_label(_("Save window geometry on exit"));
+    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(thing), gui_settings.save_geometry);
     gtk_box_pack_start(GTK_BOX(vbox1), thing, FALSE, TRUE, 0);
     gtk_widget_show(thing);
     gtk_signal_connect(GTK_OBJECT(thing), "toggled",
-		       GTK_SIGNAL_FUNC(gui_settings_use_aa_canvas_toggled), NULL);
-    gtk_tooltips_set_tip(tooltips, thing, _("You need to restart SoundTracker for this change to come into effect."), NULL);
+		       GTK_SIGNAL_FUNC(gui_settings_save_geometry_toggled), NULL);
 
     gui_subs_set_slider_value(&prefs_scopesfreq_slider, gui_settings.scopes_update_freq);
     gui_subs_set_slider_value(&prefs_trackerfreq_slider, gui_settings.tracker_update_freq);
@@ -564,20 +553,24 @@ gui_settings_load_config (void)
 
     f = prefs_open_read("settings");
     if(f) {
+	prefs_get_int(f, "st-window-x", &gui_settings.st_window_x);
+	prefs_get_int(f, "st-window-y", &gui_settings.st_window_y);
+	prefs_get_int(f, "st-window-w", &gui_settings.st_window_w);
+	prefs_get_int(f, "st-window-h", &gui_settings.st_window_h);
+
 	prefs_get_int(f, "gui-use-hexadecimal-numbers", &gui_settings.tracker_hexmode);
 	prefs_get_int(f, "gui-use-upper-case", &gui_settings.tracker_upcase);
 	prefs_get_int(f, "gui-advance-cursor-in-fx-columns", &gui_settings.advance_cursor_in_fx_columns);
 	prefs_get_int(f, "gui-asynchronous-editing", &gui_settings.asynchronous_editing);
 	prefs_get_string(f, "gui-tracker-line-format", gui_settings.tracker_line_format);
-	prefs_get_int(f, "gui-channel-numbering", &gui_settings.channel_numbering);
 	prefs_get_int(f, "gui-tempo-bpm-update", &gui_settings.tempo_bpm_update);
 	prefs_get_int(f, "gui-auto-switch", &gui_settings.auto_switch);
 	prefs_get_int(f, "gui-display-scopes", &gui_settings.gui_display_scopes);
 	prefs_get_int(f, "gui-use-backing-store", &gui_settings.gui_use_backing_store);
-	prefs_get_int(f, "gui-use-aa-canvas", &gui_settings.gui_use_aa_canvas);
 	prefs_get_int(f, "tracker-highlight-rows", &gui_settings.highlight_rows);
 	prefs_get_int(f, "tracker-highlight-rows-n", &gui_settings.highlight_rows_n);
 	prefs_get_int(f, "tracker-highlight-rows-minor-n", &gui_settings.highlight_rows_minor_n);
+	prefs_get_int(f, "save-geometry", &gui_settings.save_geometry);
 	prefs_get_int(f, "save-settings-on-exit", &gui_settings.save_settings_on_exit);
 	prefs_get_int(f, "tracker-update-frequency", &gui_settings.tracker_update_freq);
 	prefs_get_int(f, "scopes-update-frequency", &gui_settings.scopes_update_freq);
@@ -597,11 +590,11 @@ gui_settings_load_config (void)
 	prefs_get_string(f, "loadinstr-path", gui_settings.loadinstr_path);
 	prefs_get_string(f, "saveinstr-path", gui_settings.saveinstr_path);
 
-        prefs_get_string(f, "rm-path", gui_settings.rm_path);
-        prefs_get_string(f, "unzip-path", gui_settings.unzip_path);
-        prefs_get_string(f, "lha-path", gui_settings.lha_path);
-        prefs_get_string(f, "gz-path", gui_settings.gz_path);
-        prefs_get_string(f, "bz2-path", gui_settings.bz2_path);
+	prefs_get_string(f, "rm-path", gui_settings.rm_path);
+	prefs_get_string(f, "unzip-path", gui_settings.unzip_path);
+	prefs_get_string(f, "lha-path", gui_settings.lha_path);
+	prefs_get_string(f, "gz-path", gui_settings.gz_path);
+	prefs_get_string(f, "bz2-path", gui_settings.bz2_path);
 
 	prefs_close(f);
     }
@@ -616,23 +609,37 @@ gui_settings_save_config (void)
     if(!f)
 	return;
 
+    if(gui_settings.save_geometry) {
+	gdk_window_get_size (mainwindow->window,
+			     &gui_settings.st_window_w,
+			     &gui_settings.st_window_h);
+	gdk_window_get_root_origin (mainwindow->window,
+				    &gui_settings.st_window_x,
+				    &gui_settings.st_window_y);
+    }
+
+    prefs_put_int(f, "st-window-x", gui_settings.st_window_x);
+    prefs_put_int(f, "st-window-y", gui_settings.st_window_y);
+    prefs_put_int(f, "st-window-w", gui_settings.st_window_w);
+    prefs_put_int(f, "st-window-h", gui_settings.st_window_h);
+
     prefs_put_int(f, "gui-use-hexadecimal-numbers", gui_settings.tracker_hexmode);
     prefs_put_int(f, "gui-use-upper-case", gui_settings.tracker_upcase);
     prefs_put_int(f, "gui-advance-cursor-in-fx-columns", gui_settings.advance_cursor_in_fx_columns);
     prefs_put_int(f, "gui-asynchronous-editing", gui_settings.asynchronous_editing);
     prefs_put_string(f, "gui-tracker-line-format", gui_settings.tracker_line_format);
-    prefs_put_int(f, "gui-channel-numbering", gui_settings.channel_numbering);
     prefs_put_int(f, "gui-tempo-bpm-update", gui_settings.tempo_bpm_update);
     prefs_put_int(f, "gui-auto-switch", gui_settings.auto_switch);
     prefs_put_int(f, "gui-display-scopes", gui_settings.gui_display_scopes);
     prefs_put_int(f, "gui-use-backing-store", gui_settings.gui_use_backing_store);
-    prefs_put_int(f, "gui-use-aa-canvas", gui_settings.gui_use_aa_canvas);
     prefs_put_int(f, "tracker-highlight-rows", gui_settings.highlight_rows);
     prefs_put_int(f, "tracker-highlight-rows-n", gui_settings.highlight_rows_n);
     prefs_put_int(f, "tracker-highlight-rows-minor-n", gui_settings.highlight_rows_minor_n);
+    prefs_put_int(f, "save-geometry", gui_settings.save_geometry);
     prefs_put_int(f, "save-settings-on-exit", gui_settings.save_settings_on_exit);
     prefs_put_int(f, "tracker-update-frequency", gui_settings.tracker_update_freq);
     prefs_put_int(f, "scopes-update-frequency", gui_settings.scopes_update_freq);
+    prefs_put_int(f, "scopes-buffer-size", gui_settings.scopes_buffer_size);
     prefs_put_int(f, "save-all-non-empty-patterns", gui_settings.save_all_patterns);
 
     prefs_close(f);
