@@ -34,15 +34,14 @@ typedef struct gui_prefs {
 
     gboolean advance_cursor_in_fx_columns;
     gboolean asynchronous_editing;
-    gboolean channel_numbering;
 
     gboolean tempo_bpm_update;
     gboolean auto_switch;
 
     gboolean gui_display_scopes;
     gboolean gui_use_backing_store;
-    gboolean gui_use_aa_canvas;
 
+    gboolean save_geometry;
     gboolean save_settings_on_exit;
 
     int tracker_update_freq;
@@ -51,6 +50,11 @@ typedef struct gui_prefs {
 
     gboolean save_all_patterns;
 
+    int st_window_x;
+    int st_window_y;
+    int st_window_w;
+    int st_window_h;
+
     gchar loadmod_path[128];
     gchar savemod_path[128];
     gchar savemodaswav_path[128];
