@@ -29,7 +29,7 @@
 #include "extspinbutton.h"
 
 static const char *status_messages[] = {
-    N_("Idle."),
+    N_("Ready."),
     N_("Playing song..."),
     N_("Playing pattern..."),
     N_("Loading module..."),
