@@ -35,6 +35,9 @@
 #ifdef USE_GNOME
 #include <gnome.h>
 #endif
+#ifndef NO_GDK_PIXBUF
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#endif
 
 #include "i18n.h"
 #include "gui.h"
@@ -67,6 +70,14 @@ GtkWidget *gui_curins_name, *gui_cursmpl_name;
 GtkWidget *mainwindow;
 ScopeGroup *scopegroup;
 
+static GtkWidget *gui_splash_window = NULL;
+#ifndef NO_GDK_PIXBUF
+static GdkPixbuf *gui_splash_logo = NULL;
+static GtkWidget *gui_splash_logo_area;
+#endif
+static GtkWidget *gui_splash_label;
+static GtkWidget *gui_splash_close_button;
+
 static gint pipetag = -1;
 static GtkWidget *mainwindow_upper_hbox, *mainwindow_second_hbox;
 static GtkWidget *notebook;
@@ -357,22 +368,11 @@ gui_handle_standard_keys (int shift,
         if(alt) {
             switch(keyval)
             {
-                case '1':
-                gtk_notebook_set_page(GTK_NOTEBOOK(notebook), 0);
+	    case '1' ... '5':
+                gtk_notebook_set_page(GTK_NOTEBOOK(notebook), keyval - '1');
                 break;
-                case '2':
-                gtk_notebook_set_page(GTK_NOTEBOOK(notebook), 1);
-                break;
-                case '3':
-                gtk_notebook_set_page(GTK_NOTEBOOK(notebook), 2);
-                break;
-                case '4':
-                gtk_notebook_set_page(GTK_NOTEBOOK(notebook), 3);
-                break;
-                case '5':
-                gtk_notebook_set_page(GTK_NOTEBOOK(notebook), 4);
-                break;
-                default:
+	    default:
+		break;
             }
 	    handled = TRUE;
         }
@@ -434,14 +434,16 @@ gui_handle_standard_keys (int shift,
             break;
 	b = GUI_ENABLED;
 	gui_play_stop();
-	if(b) {
-            /* toggle editing mode (only if we haven't been in playing mode) */
-            gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(editing_toggle), !GUI_EDITING);
-            if (GTK_TOGGLE_BUTTON(editing_toggle)->active)
-		show_editmode_status();
-            else
-		statusbar_update(STATUS_IDLE, FALSE);
+    if(notebook_current_page != NOTEBOOK_PAGE_SAMPLE_EDITOR && notebook_current_page != NOTEBOOK_PAGE_INSTRUMENT_EDITOR) {
+		if(b) {
+        	    /* toggle editing mode (only if we haven't been in playing mode) */
+	            gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(editing_toggle), !GUI_EDITING);
+    	        if (GTK_TOGGLE_BUTTON(editing_toggle)->active)
+					show_editmode_status();
+	            else
+					statusbar_update(STATUS_IDLE, FALSE);
         }
+    }
 	handled = TRUE;
 	break;
     case GDK_Escape:
@@ -1220,20 +1222,175 @@ gui_auto_switch_page (void)
                               1);
 }
 
-int
-gui_init (int argc,
-	  char *argv[])
+#ifndef NO_GDK_PIXBUF
+static gint
+gui_splash_logo_expose (GtkWidget *widget,
+			GdkEventExpose *event,
+			gpointer data)
 {
-    GtkWidget *thing, *mainvbox, *table, *hbox, *frame, *mainvbox0;
-    GdkColormap *colormap;
-#ifdef USE_GNOME
-    GtkWidget *dockitem;
+    gdk_pixbuf_render_to_drawable (gui_splash_logo,
+				   widget->window,
+				   widget->style->black_gc,
+				   event->area.x, event->area.y,
+				   event->area.x, event->area.y,
+				   event->area.width, event->area.height,
+				   GDK_RGB_DITHER_NORMAL,
+				   0, 0);
+
+    return TRUE;
+}
+#endif
+
+static void
+gui_splash_close (void)
+{
+    gtk_widget_destroy(gui_splash_window);
+    gui_splash_window = NULL;
+}
+
+static void
+gui_splash_set_label (const gchar *text,
+		      gboolean update)
+{
+    char buf[256];
+
+    strcpy(buf, "SoundTracker v" VERSION " - ");
+    strncat(buf, text, 255-sizeof(buf));
+
+    gtk_label_set_text(GTK_LABEL(gui_splash_label), buf);
+
+    while(update && gtk_events_pending ()) {
+	gtk_main_iteration ();
+    }
+}
+
+int
+gui_splash (int argc,
+	    char *argv[])
+{
+    GtkWidget *vbox, *thing;
+#ifndef NO_GDK_PIXBUF
+    GtkWidget *hbox, *logo_area, *frame;
 #endif
 
 #ifdef USE_GNOME
     gnome_init("SoundTracker", VERSION, argc, argv);
 #else
     gtk_init(&argc, &argv);
+    gdk_rgb_init();
+#endif
+
+    gui_splash_window = gtk_window_new (GTK_WINDOW_DIALOG);
+
+    gtk_window_set_title (GTK_WINDOW(gui_splash_window), _("SoundTracker Startup"));
+//    gtk_window_set_wmclass (GTK_WINDOW (gui_splash_window), "soundtracker_startup", "SoundTracker");
+    gtk_window_set_position (GTK_WINDOW (gui_splash_window), GTK_WIN_POS_CENTER);
+    gtk_window_set_policy (GTK_WINDOW (gui_splash_window), FALSE, FALSE, FALSE);
+    gtk_window_set_modal(GTK_WINDOW(gui_splash_window), TRUE);
+
+    gtk_signal_connect(GTK_OBJECT (gui_splash_window), "delete_event",
+		       GTK_SIGNAL_FUNC (gui_splash_close), NULL);
+
+    vbox = gtk_vbox_new (FALSE, 4);
+    gtk_container_add (GTK_CONTAINER (gui_splash_window), vbox);
+
+    gtk_container_border_width(GTK_CONTAINER(vbox), 4);
+
+    /* Show splash screen if enabled and image available. */
+
+#ifndef NO_GDK_PIXBUF
+    gui_splash_logo = gdk_pixbuf_new_from_file(PREFIX"/share/soundtracker/soundtracker_splash.png");
+    if(gui_splash_logo) {
+	thing = gtk_hseparator_new();
+	gtk_widget_show(thing);
+	gtk_box_pack_start(GTK_BOX(vbox), thing, FALSE, TRUE, 0);
+
+	hbox = gtk_hbox_new(FALSE, 4);
+	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	gtk_widget_show(hbox);
+
+	add_empty_vbox(hbox);
+
+	frame = gtk_frame_new(NULL);
+	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
+	gtk_box_pack_start(GTK_BOX(hbox), frame, FALSE, TRUE, 0);
+	gtk_widget_show(frame);
+
+	gui_splash_logo_area = logo_area = gtk_drawing_area_new ();
+	gtk_container_add (GTK_CONTAINER(frame), logo_area);
+	gtk_widget_show(logo_area);
+
+	add_empty_vbox(hbox);
+
+	gtk_signal_connect (GTK_OBJECT (logo_area), "expose_event",
+			    GTK_SIGNAL_FUNC (gui_splash_logo_expose),
+			    NULL);
+
+	gtk_drawing_area_size (GTK_DRAWING_AREA (logo_area),
+			       gdk_pixbuf_get_width(gui_splash_logo),
+			       gdk_pixbuf_get_height(gui_splash_logo));
+    }
+#endif
+
+    /* Show version number. */
+
+    thing = gtk_hseparator_new();
+    gtk_widget_show(thing);
+    gtk_box_pack_start(GTK_BOX(vbox), thing, FALSE, TRUE, 0);
+
+    gui_splash_label = thing = gtk_label_new("");
+    gtk_box_pack_start(GTK_BOX(vbox), thing, FALSE, TRUE, 0);
+    gtk_widget_show(thing);
+    gui_splash_set_label(_("Loading..."), FALSE);
+
+    /* Show tips if enabled. */
+
+    if(tips_dialog_show_tips) {
+	GtkWidget *tipsbox = tips_dialog_get_vbox();
+
+	if(tipsbox) {
+	    thing = gtk_hseparator_new();
+	    gtk_widget_show(thing);
+	    gtk_box_pack_start(GTK_BOX(vbox), thing, FALSE, TRUE, 0);
+
+	    gtk_box_pack_start(GTK_BOX(vbox), tipsbox, FALSE, FALSE, 0);
+	    gtk_widget_show(tipsbox);
+	}
+    }
+
+    thing = gtk_hseparator_new();
+    gtk_widget_show(thing);
+    gtk_box_pack_start(GTK_BOX(vbox), thing, FALSE, TRUE, 0);
+
+    if(
+#ifndef NO_GDK_PIXBUF
+       gui_splash_logo ||
+#endif
+       tips_dialog_show_tips) {
+	gui_splash_close_button = thing = gtk_button_new_with_label(_("Use SoundTracker!"));
+	gtk_widget_show(thing);
+	gtk_box_pack_start(GTK_BOX(vbox), thing, FALSE, TRUE, 0);
+	gtk_signal_connect(GTK_OBJECT (thing), "clicked",
+			   GTK_SIGNAL_FUNC(gui_splash_close), NULL);
+	gtk_widget_set_sensitive(thing, FALSE);
+    }
+
+    gtk_widget_show(vbox);
+    gtk_widget_show(gui_splash_window);
+
+    gui_splash_set_label(_("Loading..."), TRUE);
+
+    return 1;
+}
+
+int
+gui_final (int argc,
+	   char *argv[])
+{
+    GtkWidget *thing, *mainvbox, *table, *hbox, *frame, *mainvbox0;
+    GdkColormap *colormap;
+#ifdef USE_GNOME
+    GtkWidget *dockitem;
 #endif
 
     pipetag = gdk_input_add(audio_backpipe, GDK_INPUT_READ, read_mixer_pipe, NULL);
@@ -1244,9 +1401,24 @@ gui_init (int argc,
     mainwindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
     gtk_window_set_title (GTK_WINDOW (mainwindow), "SoundTracker " VERSION);
 #endif
+
     gtk_signal_connect (GTK_OBJECT (mainwindow), "delete_event",
 			GTK_SIGNAL_FUNC (menubar_quit_requested), NULL);
 
+    if(gui_splash_window) {
+	gtk_window_set_transient_for(GTK_WINDOW(gui_splash_window),
+				     GTK_WINDOW(mainwindow));
+    }
+
+    if(gui_settings.st_window_x != -666) {
+	gtk_window_set_default_size (GTK_WINDOW (mainwindow),
+				     gui_settings.st_window_w,
+				     gui_settings.st_window_h);
+	gtk_widget_set_uposition (GTK_WIDGET (mainwindow),
+				  gui_settings.st_window_x,
+				  gui_settings.st_window_y);
+    }
+
     fileops_dialogs[DIALOG_LOAD_MOD] = file_selection_create(_("Load XM..."), file_selected);
     gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileops_dialogs[DIALOG_LOAD_MOD]), gui_settings.loadmod_path);
     fileops_dialogs[DIALOG_SAVE_MOD] = file_selection_create(_("Save XM..."), file_selected);
@@ -1537,6 +1709,8 @@ gui_init (int argc,
 
     /* Status Bar */
 
+#define WELCOME_MESSAGE _("Welcome to SoundTracker!")
+
 #ifdef USE_GNOME
     dockitem = gnome_dock_item_new("Status Bar", (GNOME_DOCK_ITEM_BEH_EXCLUSIVE | GNOME_DOCK_ITEM_BEH_NEVER_VERTICAL));
     gnome_app_add_dock_item(GNOME_APP(mainwindow), GNOME_DOCK_ITEM(dockitem), GNOME_DOCK_BOTTOM, 0, 0, 0);
@@ -1595,13 +1769,34 @@ gui_init (int argc,
 
     gtk_widget_show (mainwindow);
 
-    if(show_tips) {
-	tips_dialog_create();
-    }
-
     if(!keys_init()) {
 	return 0;
     }
 
+    if(gui_splash_window) {
+	if(
+#ifndef NO_GDK_PIXBUF
+           gui_splash_logo ||
+#endif
+           tips_dialog_show_tips) {
+	    gdk_window_raise(gui_splash_window->window);
+//	    gtk_window_set_transient_for(GTK_WINDOW(gui_splash_window), GTK_WINDOW(mainwindow));
+// (doesn't do anything on WindowMaker)
+	    gui_splash_set_label(_("Ready."), TRUE);
+#ifndef NO_GDK_PIXBUF
+	    if(gui_splash_logo) {
+		gtk_widget_add_events(gui_splash_logo_area,
+				      GDK_BUTTON_PRESS_MASK);
+		gtk_signal_connect (GTK_OBJECT (gui_splash_logo_area), "button_press_event",
+				    GTK_SIGNAL_FUNC (gui_splash_close),
+				    NULL);
+	    }
+#endif
+	    gtk_widget_set_sensitive(gui_splash_close_button, TRUE);
+	} else {
+	    gui_splash_close();
+	}
+    }
+
     return 1;
 }
