@@ -53,15 +53,14 @@ extern int notebook_current_page; /* one of the above */
 #define GUI_ENABLED (gui_playing_mode != PLAYING_SONG && gui_playing_mode != PLAYING_PATTERN)
 #define GUI_EDITING (GTK_TOGGLE_BUTTON(editing_toggle)->active)
 
-#define WELCOME_MESSAGE _("Welcome to Sound Tracker !")
-
 extern GtkWidget *editing_toggle;
 extern GtkWidget *gui_curins_name, *gui_cursmpl_name;
 extern GtkWidget *mainwindow;
 
 extern void show_editmode_status (void);
 
-int                  gui_init                         (int argc, char *argv[]);
+int                  gui_splash                       (int argc, char *argv[]);
+int                  gui_final                        (int argc, char *argv[]);
 
 void                 gui_playlist_initialize          (void);
 
