@@ -6,7 +6,8 @@
  * it's the solution I've come up with after nearly three years of
  * experimentation...
  *
- * Copyright (C) 1997-1999 Michael Krause
+ * Copyright (C) 1997-2001 Michael Krause
+ * Copyright (C) 2000 Fabian Giesen (Win32 stuff)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +26,8 @@
 
 #include <config.h>
 
+#if !defined(_WIN32)
+
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>
@@ -129,24 +132,25 @@ typedef struct keys_group {
     const char *title;
     const char *explanation;
     keys_key *keys;
+    keys_key *keys_edit;
 } keys_group;
 
 static keys_group groups[] = {
-    { N_("Upper Octave Keys.."),
+    { N_("Upper Octave Keys..."),
       N_("These are the keys on the upper half of the keyboard. "
       "The c key is normally the key to the right of the TAB key. "
       "The rest of the keys should be ordered in a piano keyboard fashion, including "
       "the number keys row above."),
       keys1
     },
-    { N_("Lower Octave Keys.."),
+    { N_("Lower Octave Keys..."),
       N_("These are the keys on the lower half of the keyboard. "
       "The c key is normally the first character key to the right of the left Shift key. "
       "The rest of the keys should be ordered in a piano keyboard fashion, including "
       "the row above."),
       keys2
     },
-    { N_("Other Keys.."),
+    { N_("Other Keys..."),
       N_("Various other keys"),
       keys3
     }
@@ -169,7 +173,80 @@ static GList *xkeys[2];
 
 static int capturing = 0, capturing_all;
 
+static gint
+keys_keys_array_length (keys_key *keys)
+{
+    int i = 0;
+
+    while((keys++)->title)
+	i++;
+
+    return i;
+}
+
+static keys_key *
+keys_duplicate_keys_array (keys_key *keys)
+{
+    int l = keys_keys_array_length(keys);
+    keys_key *copy;
+
+    copy = g_new(keys_key, l + 1);
+    memcpy(copy, keys, l * sizeof(keys_key));
+    copy[l].title = NULL;
+
+    return copy;
+}
+
 static void
+keys_initialize_editing (void)
+{
+    int i;
+
+    for(i = 0; i < NUM_KEY_GROUPS; i++) {
+	groups[i].keys_edit = keys_duplicate_keys_array(groups[i].keys);
+	g_assert(groups[i].keys_edit != NULL);
+    }
+}
+
+static void
+keys_finish_editing (void)
+{
+    int i;
+
+    for(i = 0; i < NUM_KEY_GROUPS; i++) {
+	g_free(groups[i].keys_edit);
+	groups[i].keys_edit = NULL;
+    }
+}
+
+static void
+keys_apply (void)
+{
+    int i;
+
+    for(i = 0; i < NUM_KEY_GROUPS; i++) {
+	g_assert(groups[i].keys_edit != NULL);
+	g_assert(groups[i].keys != NULL);
+	memcpy(groups[i].keys, groups[i].keys_edit, keys_keys_array_length(groups[i].keys) * sizeof(keys_key));
+    }
+}
+
+static void
+keys_cancel (void)
+{
+    gtk_widget_destroy(configwindow);
+    configwindow = NULL;
+    keys_finish_editing();
+}
+
+static void
+keys_ok (void)
+{
+    keys_apply();
+    keys_cancel();
+}
+
+static gboolean
 keys_encode_assignment (gchar *string,
 			int modifiers,
 			int xkeysym)
@@ -189,12 +266,13 @@ keys_encode_assignment (gchar *string,
 	for(k = 0; k < xkeymaplen; k++) {
 	    if(xkeymap[k].xname && xkeymap[k].xkeysym == xkeysym) {
 		strcat(string, xkeymap[k].xname);
-		return;
+		return TRUE;
 	    }
 	}
     }
 
     strcat(string, NONE_TEXT);
+    return FALSE;
 }
 
 static int
@@ -251,7 +329,7 @@ keys_key_group_changed (void *a,
 
     gtk_clist_clear(GTK_CLIST(cw_clist));
     gtk_clist_freeze(GTK_CLIST(cw_clist));
-    for(kpt = groups[n].keys; kpt->title; kpt++) {
+    for(kpt = groups[n].keys_edit; kpt->title; kpt++) {
 	insertbuf[0] = (gchar*)gettext(kpt->title);
 	keys_encode_assignment(string, kpt->modifiers, kpt->xkeysym);
 	gtk_clist_append(GTK_CLIST(cw_clist), insertbuf);
@@ -264,8 +342,8 @@ keys_key_group_changed (void *a,
 
 static void
 keys_clist_select (GtkCList *list,
-		       gint row,
-		       gint column)
+		   gint row,
+		   gint column)
 {
     gchar *code;
     int mod, i, h;
@@ -276,7 +354,7 @@ keys_clist_select (GtkCList *list,
     cw_currentkey = -1;
 
     // Set explanation
-    gtk_label_set_text(GTK_LABEL(cw_explabel2), gettext(groups[cw_currentgroup].keys[row].explanation));
+    gtk_label_set_text(GTK_LABEL(cw_explabel2), gettext(groups[cw_currentgroup].keys_edit[row].explanation));
 
     // Set combo box list
     if(cw_combostrings != (0)) {
@@ -285,14 +363,14 @@ keys_clist_select (GtkCList *list,
     }
 
     // Set modifier toggles
-    mod = groups[cw_currentgroup].keys[row].modifiers;
+    mod = groups[cw_currentgroup].keys_edit[row].modifiers;
     for(i = 0; i <= 2; i++) {
 	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(cw_modtoggles[i]), mod & (1 << i));
     }
     
     // Set combo box entry
     code = NONE_TEXT;
-    h = groups[cw_currentgroup].keys[row].xkeysym;
+    h = groups[cw_currentgroup].keys_edit[row].xkeysym;
     for(i = 0; h != 0 && i < xkeymaplen; i++) {
 	if(xkeymap[i].xkeysym == h) {
 	    code = xkeymap[i].xname;
@@ -313,7 +391,7 @@ keys_assignment_changed (void)
     if(cw_currentgroup == -1 || cw_currentkey == -1)
 	return;
 
-    groups[cw_currentgroup].keys[cw_currentkey].modifiers = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cw_modtoggles[0]))
+    groups[cw_currentgroup].keys_edit[cw_currentkey].modifiers = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cw_modtoggles[0]))
 	+ 2 * gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cw_modtoggles[1]))
 	+ 4 * gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cw_modtoggles[2]));
 
@@ -325,10 +403,10 @@ keys_assignment_changed (void)
 	}
     }
 
-    groups[cw_currentgroup].keys[cw_currentkey].xkeysym = keysym;
+    groups[cw_currentgroup].keys_edit[cw_currentkey].xkeysym = keysym;
 
     keys_encode_assignment(string,
-			   groups[cw_currentgroup].keys[cw_currentkey].modifiers,
+			   groups[cw_currentgroup].keys_edit[cw_currentkey].modifiers,
 			   keysym);
     gtk_clist_set_text(GTK_CLIST(cw_clist), cw_currentkey, 1, string);
 }
@@ -371,14 +449,17 @@ keys_keyevent (GtkWidget *widget,
 	    + 2 * ((event->state & GDK_CONTROL_MASK) != 0)
 	    + 4 * ((event->state & GDK_MOD1_MASK) != 0);
 
-	groups[cw_currentgroup].keys[cw_currentkey].xkeysym = keysym;
-	groups[cw_currentgroup].keys[cw_currentkey].modifiers = mod;
-	keys_encode_assignment(string, mod, keysym);
+	/* If this doesn't succeed, the keymap contains logical errors */
+	g_assert(keys_encode_assignment(string, mod, keysym));
+
+	groups[cw_currentgroup].keys_edit[cw_currentkey].xkeysym = keysym;
+	groups[cw_currentgroup].keys_edit[cw_currentkey].modifiers = mod;
+
 	gtk_clist_set_text(GTK_CLIST(cw_clist), cw_currentkey, 1, string);
 
 	if(capturing_all) {
 	    int nextkey = cw_currentkey + 1;
-	    if(groups[cw_currentgroup].keys[nextkey].title) {
+	    if(groups[cw_currentgroup].keys_edit[nextkey].title) {
 		gtk_clist_select_row(GTK_CLIST(cw_clist), nextkey, 0);
 		gtk_clist_moveto(GTK_CLIST(cw_clist), nextkey, 0, 0.5, 0.0);
 	    } else {
@@ -416,17 +497,10 @@ keys_learn_all_keys_clicked (void)
     keys_lb_switch(0);
 }
 
-static void
-keys_close_requested (void)
-{
-    gtk_widget_destroy(configwindow);
-    configwindow = NULL;
-}
-
 void
 keys_dialog (void)
 {
-    GtkWidget *mainbox, *box1, *box2, *box3, *box4, *thing, *frame;
+    GtkWidget *mainbox, *box1, *box2, *box3, *box4, *thing, *frame, *hbox;
     OptionMenuItem menu1[NUM_KEY_GROUPS];
     int i;
     gchar *listtitles[2] = {
@@ -446,7 +520,7 @@ keys_dialog (void)
     gtk_window_set_title(GTK_WINDOW(configwindow), _("Keyboard Configuration"));
 #endif
     gtk_signal_connect (GTK_OBJECT (configwindow), "delete_event",
-			GTK_SIGNAL_FUNC (keys_close_requested), NULL);
+			GTK_SIGNAL_FUNC (keys_cancel), NULL);
 
     mainbox = gtk_vbox_new(FALSE, 2);
     gtk_container_border_width(GTK_CONTAINER(mainbox), 4);
@@ -457,12 +531,16 @@ keys_dialog (void)
 #endif
     gtk_widget_show(mainbox);
 
+    keys_initialize_editing();
+    capturing = 0;
+    capturing_all = 0;
+
     // Key Group Selector
     for(i = 0; i < NUM_KEY_GROUPS; i++) {
 	menu1[i].name = gettext(groups[i].title);
 	menu1[i].func = keys_key_group_changed;
     }
-    thing = build_option_menu(menu1, NUM_KEY_GROUPS, 0);
+    thing = gui_build_option_menu(menu1, NUM_KEY_GROUPS, 0);
     gtk_widget_show(thing);
     gtk_box_pack_start(GTK_BOX(mainbox), thing, FALSE, FALSE, 0);
 
@@ -472,7 +550,7 @@ keys_dialog (void)
     gtk_box_pack_start(GTK_BOX(mainbox), box1, TRUE, TRUE, 0);
 
     // List at the left side of the window
-    thing = get_clist_in_scrolled_window(2, listtitles, box1);
+    thing = gui_clist_in_scrolled_window(2, listtitles, box1);
     gtk_clist_set_selection_mode(GTK_CLIST(thing), GTK_SELECTION_BROWSE);
     gtk_clist_column_titles_passive(GTK_CLIST(thing));
     gtk_clist_set_column_justification(GTK_CLIST(thing), 0, GTK_JUSTIFY_LEFT);
@@ -587,6 +665,50 @@ keys_dialog (void)
     gtk_signal_connect(GTK_OBJECT(configwindow), "button_press_event",
 		       GTK_SIGNAL_FUNC(keys_buttonevent), NULL);
 
+
+    /* The button area */
+    thing = gtk_hseparator_new();
+    gtk_widget_show(thing);
+    gtk_box_pack_start(GTK_BOX(mainbox), thing, FALSE, TRUE, 0);
+
+    hbox = gtk_hbutton_box_new ();
+    gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbox), 4);
+    gtk_button_box_set_layout (GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_END);
+//    gtk_button_box_set_child_ipadding (GTK_BUTTON_BOX(hbox), 30, 0);
+    gtk_box_pack_start (GTK_BOX (mainbox), hbox,
+			FALSE, FALSE, 0);
+    gtk_widget_show (hbox);
+
+#ifdef USE_GNOME
+    thing = gnome_stock_button (GNOME_STOCK_BUTTON_OK);
+#else
+    thing = gtk_button_new_with_label (_ ("Ok"));
+#endif
+    gtk_signal_connect (GTK_OBJECT (thing), "clicked",
+			GTK_SIGNAL_FUNC (keys_ok), NULL);
+    gtk_box_pack_start (GTK_BOX (hbox), thing, FALSE, FALSE, 0);
+    gtk_widget_show (thing);
+
+#ifdef USE_GNOME
+    thing = gnome_stock_button (GNOME_STOCK_BUTTON_APPLY);
+#else
+    thing = gtk_button_new_with_label (_ ("Apply"));
+#endif
+    gtk_signal_connect (GTK_OBJECT (thing), "clicked",
+			GTK_SIGNAL_FUNC (keys_apply), NULL);
+    gtk_box_pack_start (GTK_BOX (hbox), thing, FALSE, FALSE, 0);
+    gtk_widget_show (thing);
+
+#ifdef USE_GNOME
+    thing = gnome_stock_button (GNOME_STOCK_BUTTON_CANCEL);
+#else
+    thing = gtk_button_new_with_label (_ ("Cancel"));
+#endif
+    gtk_signal_connect (GTK_OBJECT (thing), "clicked",
+			GTK_SIGNAL_FUNC (keys_cancel), NULL);
+    gtk_box_pack_start (GTK_BOX (hbox), thing, FALSE, FALSE, 0);
+    gtk_widget_show (thing);
+
     gtk_widget_show (configwindow);
 }
 
@@ -601,16 +723,18 @@ keys_load_config (void)
 {
     char buf[256];
     FILE *f;
+    prefs_node *p;
     int r = 0;
     int g;
     keys_key *k;
     int keysym, keycode, mod;
 
-    sprintf(buf, "%s/keyboard", prefs_get_prefsdir());
-
-    f = fopen(buf, "rb");
-    if(!f)
+    p = prefs_open_read("keyboard");
+    if(!p) {
 	return 0;
+    }
+
+    f = prefs_get_file_pointer(p);
 
     // Oh, oh, oh my god... goto is sooo evil :)
     while(!feof(f)) {
@@ -645,7 +769,7 @@ keys_load_config (void)
     r = 1;
 
   err:
-    fclose(f);
+    prefs_close(p);
     return r;
 }
 
@@ -654,15 +778,16 @@ keys_save_config (void)
 {
     char buf[256];
     FILE *f;
+    prefs_node *p;
     int g;
     keys_key *k;
 
-    prefs_check_prefs_dir();
-    sprintf(buf, "%s/keyboard", prefs_get_prefsdir());
-
-    f = fopen(buf, "wb");
-    if(!f)
+    p = prefs_open_write("keyboard");
+    if(!p) {
 	return 0;
+    }
+
+    f = prefs_get_file_pointer(p);
 
     for(g = 0; g < NUM_KEY_GROUPS; g++) {
 	for(k = groups[g].keys; k->title; k++) {
@@ -673,7 +798,7 @@ keys_save_config (void)
 	}
     }
 
-    fclose(f);
+    prefs_close(p);
     return 1;
 }
 
@@ -785,6 +910,45 @@ keys_make_xkeys (void)
     g_free(keyname);
 }
 
+static void
+keys_fixup_xkeymap (void)
+{
+    /* The problem are custom keymaps that contain lines such as:
+           keycode 0x18 =  Q
+       instead of a correct
+           keycode 0x18 =  q Q
+       Fix these.
+    */
+
+    int i;
+    KeySym k1, k2;
+    gchar a[2] = { 0, 0 };
+
+    if(symspercode < 2)
+	return;
+
+    for(i = 0; i < xkeymaplen / symspercode; i++) {
+	k1 = xkeymap[i * symspercode + 0].xkeysym;
+	k2 = xkeymap[i * symspercode + 1].xkeysym;
+
+	if(k2 == 0) {
+	    if(k1 >= 'A' && k1 <= 'Z') {
+		fprintf(stderr, "*** keys_fixup_xkeymap: %c -> ", (int)k1);
+		k2 = k1;
+		k1 += 'a' - 'A';
+		fprintf(stderr, "%c %c\n", (int)k1, (int)k2);
+
+		a[0] = k1;
+		xkeymap[i * symspercode + 1].xname = xkeymap[i * symspercode + 0].xname;
+		xkeymap[i * symspercode + 0].xname = g_strdup(a);
+
+		xkeymap[i * symspercode + 0].xkeysym = k1;
+		xkeymap[i * symspercode + 1].xkeysym = k2;
+	    }
+    	}
+    }
+}
+
 int
 keys_init (void)
 {
@@ -840,6 +1004,7 @@ keys_init (void)
 
     XFree(servsyms);
 
+    keys_fixup_xkeymap();
     keys_make_xkeys();
 
     if(!keys_load_config()) {
@@ -895,3 +1060,69 @@ keys_is_key_pressed (guint32 keysym,
 
     return FALSE;
 }
+
+#else /* !defined(_WIN32) */
+
+#include <stdio.h>
+#include <ctype.h>
+#include <string.h>
+#include <stdlib.h>
+
+#include <glib.h>
+//#include <gdk/gdkx.h>
+//#include "X11/Xlib.h"
+//#include <X11/keysym.h>
+
+#include <gtk/gtk.h>
+#include <glib.h>
+
+#include "i18n.h"
+#include "keys.h"
+#include "gui-subs.h"
+#include "gui.h"
+#include "menubar.h"
+#include "preferences.h"
+
+char *lowkey="<>aAyYsSxXcCfFvVgGbBhHnNmMkK,;lL.:";
+
+void
+keys_dialog (void)
+{
+}
+
+int
+keys_save_config (void)
+{
+    return 1;
+}
+
+int
+keys_init (void)
+{
+    return 1;
+}
+
+guint32
+keys_get_key_meaning (guint32 keysym,
+		      int modifiers)
+
+{
+    char *c;
+
+    for (c=lowkey; *c; c++)
+      if (*c==keysym)
+        return (c-lowkey)>>1;
+
+    return -1;
+}
+
+/* Yeah! Let's fake around X's stupid auto-repeat! X sends a
+   KeyRelease before the KeyPress event of an auto-repeat. */
+gboolean
+keys_is_key_pressed (guint32 keysym,
+		     int modifiers)
+{
+    return FALSE;
+}
+
+#endif /* !defined(_WIN32) */
