@@ -1,7 +1,8 @@
+
 /*
  * The Real SoundTracker - GUI (menu bar)
  *
- * Copyright (C) 1999-2001 Michael Krause
+ * Copyright (C) 1999-2002 Michael Krause
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,6 +23,11 @@
 
 #include <string.h>
 
+#ifndef USE_GNOME
+#include "X11/Xlib.h"
+#include <gdk/gdkkeysyms.h>
+#endif
+
 #include "i18n.h"
 #include "menubar.h"
 #include "gui.h"
@@ -48,9 +54,7 @@
 #else
 static GtkItemFactory *item_factory;
 static GtkAccelGroup *accel_group;
-#ifdef ENABLE_NLS
-static int all_translated;
-#endif
+static GtkItemFactoryEntry *menubar_gtk_items = NULL;
 #endif
 
 static gboolean mark_mode_toggle_ignore = FALSE;
@@ -72,10 +76,10 @@ static void
 about_dialog (void)
 {
 #ifdef USE_GNOME
-    const gchar *authors[] = {"Michael Krause <m.krause@tu-harburg.de>", NULL};
+    const gchar *authors[] = {"Michael Krause <rawstyle@soundtracker.org>", NULL};
     GtkWidget *about = gnome_about_new("SoundTracker",
 				       VERSION,
-				       "Copyright (C) 1998-2001 Michael Krause",
+				       "Copyright (C) 1998-2002 Michael Krause",
 				       authors,
 				       "Includes OpenCP player from Niklas Beisert and Tammo Hinrichs.",
 				       NULL);
@@ -90,7 +94,7 @@ about_dialog (void)
     gtk_window_position (GTK_WINDOW(about), GTK_WIN_POS_CENTER);
     gtk_window_set_title(GTK_WINDOW(about), "About SoundTracker");
 
-    label = gtk_label_new("SoundTracker " VERSION "\n\nCopyright (C) 1998-2001 Michael Krause\n<m.krause@tu-harburg.de>"
+    label = gtk_label_new("SoundTracker " VERSION "\n\nCopyright (C) 1998-2002 Michael Krause\n<rawstyle@soundtracker.org>"
 			  "\n\nIncludes OpenCP player from Niklas Beisert and Tammo Hinrichs.");
     gtk_container_border_width(GTK_CONTAINER(GTK_DIALOG(about)->vbox), 10);
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(about)->vbox), label, TRUE, TRUE, 10);
@@ -231,9 +235,13 @@ menubar_handle_edit_menu (void *p,
         track_editor_cut_selection,                     // 10
         track_editor_copy_selection,                    // 11
         track_editor_paste_selection,                   // 12
-	track_editor_kill_notes_track,                   // 13
+	track_editor_kill_notes_track,                  // 13
         NULL,                                           // 14
-        NULL                                            // 15
+        NULL,                                           // 15
+        NULL,                                           // 16
+        NULL,                                           // 17
+        NULL,                                           // 18
+        NULL,                                           // 19
     };
 
 #ifdef USE_GNOME
@@ -255,6 +263,18 @@ menubar_handle_edit_menu (void *p,
     case 15:
         track_editor_cmd_mvalue(t, FALSE);  /* decrement CMD value */
         break;
+    case 16:
+	transposition_transpose_selection(t, +1);
+	break;
+    case 17:
+	transposition_transpose_selection(t, -1);
+	break;
+    case 18:
+	transposition_transpose_selection(t, +12);
+	break;
+    case 19:
+	transposition_transpose_selection(t, -12);
+	break;
 
     default:
         functions[a](t);
@@ -274,7 +294,50 @@ menubar_settings_tracker_prev_font (void)
     trackersettings_cycle_font_backward(TRACKERSETTINGS(trackersettings));
 }
 
-#ifdef USE_GNOME
+#ifndef USE_GNOME
+
+/* Define GNOME stuff for our GNOME->GtkItemFactory converter. */
+
+typedef struct GnomeUIInfo {
+    int type;
+    gchar *title;
+    gpointer dummy1;
+    gpointer func;
+    gpointer funcparam;
+    gpointer dummy2;
+    int dummy3, dummy4;
+    int shortcut;
+    int shortcutmod;
+    gpointer dummy5;
+    GtkWidget *widget;
+} GnomeUIInfo;
+
+#define GNOME_APP_UI_END 0
+#define GNOME_APP_UI_ITEM 1
+#define GNOME_APP_UI_SEPARATOR 2
+#define GNOME_APP_UI_TOGGLEITEM 3
+#define GNOME_APP_UI_SUBTREE 4
+
+#define GNOME_APP_PIXMAP_STOCK 0
+#define GNOME_APP_PIXMAP_NONE 0
+
+#define GNOME_STOCK_MENU_OPEN 0
+#define GNOME_STOCK_MENU_SAVE_AS 0
+#define GNOME_STOCK_MENU_QUIT 0
+#define GNOME_STOCK_MENU_NEW 0
+#define GNOME_STOCK_MENU_CUT 0
+#define GNOME_STOCK_MENU_COPY 0
+#define GNOME_STOCK_MENU_PASTE 0
+#define GNOME_STOCK_MENU_PREF 0
+#define GNOME_STOCK_MENU_SAVE 0
+#define GNOME_STOCK_MENU_ABOUT 0
+#define GNOME_STOCK_MENU_BOOK_RED 0
+
+#define GNOMEUIINFO_SEPARATOR { GNOME_APP_UI_SEPARATOR, "-", }
+#define GNOMEUIINFO_END { GNOME_APP_UI_END, }
+#define GNOMEUIINFO_SUBTREE(X,Y) { GNOME_APP_UI_SUBTREE, (X), (Y) }
+
+#endif
 
 static GnomeUIInfo file_menu[] = {
     { GNOME_APP_UI_ITEM, N_("_Open..."), NULL, fileops_open_dialog, (gpointer)DIALOG_LOAD_MOD, NULL,
@@ -303,6 +366,9 @@ static GnomeUIInfo module_menu[] = {
       GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_NEW, 0, 0, NULL },
     { GNOME_APP_UI_ITEM, N_("Clear _Patterns Only"), NULL, menubar_clear_clicked, (gpointer)1, NULL,
       GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_NEW, 0, 0, NULL },
+
+    GNOMEUIINFO_SEPARATOR,
+
     { GNOME_APP_UI_ITEM, N_("_Optimize Module"), NULL, modinfo_optimize_module, (gpointer)0, NULL,
       GNOME_APP_PIXMAP_NONE, 0, 0, 0, NULL },
 
@@ -327,12 +393,21 @@ static GnomeUIInfo edit_track_menu[] = {
       GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_COPY, GDK_F4, GDK_SHIFT_MASK, NULL },
     { GNOME_APP_UI_ITEM, N_("_Paste"), NULL, menubar_handle_edit_menu, (gpointer)5, NULL,
       GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PASTE, GDK_F5, GDK_SHIFT_MASK, NULL },
+
+    GNOMEUIINFO_SEPARATOR,
+
     { GNOME_APP_UI_ITEM, N_("_Kill notes"), NULL, menubar_handle_edit_menu, (gpointer)13, NULL,
-      GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_NEW, GDK_K, GDK_CONTROL_MASK, NULL },
-    { GNOME_APP_UI_ITEM, N_("_Insert"), NULL, menubar_handle_edit_menu, (gpointer)6, NULL,
+      GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_NEW, 'K', GDK_CONTROL_MASK, NULL },
+
+    GNOMEUIINFO_SEPARATOR,
+
+    { GNOME_APP_UI_ITEM, N_("_Insert track"), NULL, menubar_handle_edit_menu, (gpointer)6, NULL,
       GNOME_APP_PIXMAP_NONE, 0, 0, 0, NULL },
-    { GNOME_APP_UI_ITEM, N_("_Delete"), NULL, menubar_handle_edit_menu, (gpointer)7, NULL,
+    { GNOME_APP_UI_ITEM, N_("_Delete track"), NULL, menubar_handle_edit_menu, (gpointer)7, NULL,
       GNOME_APP_PIXMAP_NONE, 0, 0, 0, NULL },
+
+    GNOMEUIINFO_SEPARATOR,
+
     { GNOME_APP_UI_ITEM, N_("Increment cmd value"), NULL, menubar_handle_edit_menu, (gpointer)14, NULL,
       GNOME_APP_PIXMAP_NONE, GNOME_STOCK_MENU_NEW, '=', GDK_CONTROL_MASK, NULL },
     { GNOME_APP_UI_ITEM, N_("Decrement cmd value"), NULL, menubar_handle_edit_menu, (gpointer)15, NULL,
@@ -346,15 +421,32 @@ static GnomeUIInfo edit_selection_menu[] = {
       GNOME_APP_PIXMAP_NONE, 0, 'B', GDK_CONTROL_MASK, NULL },
     { GNOME_APP_UI_ITEM, N_("C_lear block marks"), NULL, menubar_handle_edit_menu, (gpointer)9, NULL,
       GNOME_APP_PIXMAP_NONE, 0, 'B', GDK_CONTROL_MASK | GDK_SHIFT_MASK, NULL },
+
+    GNOMEUIINFO_SEPARATOR,
+
     { GNOME_APP_UI_ITEM, N_("C_ut"), NULL, menubar_handle_edit_menu, (gpointer)10, NULL,
       GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CUT, 'X', GDK_CONTROL_MASK, NULL },
     { GNOME_APP_UI_ITEM, N_("_Copy"), NULL, menubar_handle_edit_menu, (gpointer)11, NULL,
       GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_COPY, 'C', GDK_CONTROL_MASK, NULL },
     { GNOME_APP_UI_ITEM, N_("_Paste"), NULL, menubar_handle_edit_menu, (gpointer)12, NULL,
       GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PASTE, 'V', GDK_CONTROL_MASK, NULL },
+
+    GNOMEUIINFO_SEPARATOR,
+
     { GNOME_APP_UI_ITEM, N_("_Interpolate effects"), NULL, menubar_handle_edit_menu, (gpointer)8, NULL,
       GNOME_APP_PIXMAP_NONE, 0, 'I', GDK_CONTROL_MASK, NULL },
 
+    GNOMEUIINFO_SEPARATOR,
+
+    { GNOME_APP_UI_ITEM, N_("Transpose half-note up"), NULL, menubar_handle_edit_menu, (gpointer)16, NULL,
+      GNOME_APP_PIXMAP_NONE, 0, 'Q', GDK_MOD1_MASK, NULL },
+    { GNOME_APP_UI_ITEM, N_("Transpose half-note down"), NULL, menubar_handle_edit_menu, (gpointer)17, NULL,
+      GNOME_APP_PIXMAP_NONE, 0, 'A', GDK_MOD1_MASK, NULL },
+    { GNOME_APP_UI_ITEM, N_("Transpose octave up"), NULL, menubar_handle_edit_menu, (gpointer)18, NULL,
+      GNOME_APP_PIXMAP_NONE, 0, 'Q', GDK_MOD1_MASK | GDK_SHIFT_MASK, NULL },
+    { GNOME_APP_UI_ITEM, N_("Transpose octave down"), NULL, menubar_handle_edit_menu, (gpointer)19, NULL,
+      GNOME_APP_PIXMAP_NONE, 0, 'A', GDK_MOD1_MASK | GDK_SHIFT_MASK, NULL },
+
     GNOMEUIINFO_END
 
 };
@@ -377,6 +469,8 @@ static GnomeUIInfo edit_menu[] = {
     GNOMEUIINFO_END
 };
 
+#ifdef USE_GNOME
+
 static GnomeUIInfo track_editor_popup_edit_selection_menu[] = {
     { GNOME_APP_UI_ITEM, N_("C_lear block marks"), NULL, menubar_handle_edit_menu, (gpointer)9, NULL,
       GNOME_APP_PIXMAP_NONE, 0, 'B', GDK_CONTROL_MASK | GDK_SHIFT_MASK, NULL },
@@ -399,11 +493,16 @@ GnomeUIInfo track_editor_popup_menu[] =	{
     GNOMEUIINFO_END
 };
 
+#endif
+
 static GnomeUIInfo pattern_menu[] = {
     { GNOME_APP_UI_ITEM, N_("_Find Unused Pattern"), NULL, modinfo_find_unused_pattern, (gpointer)0, NULL,
       GNOME_APP_PIXMAP_NONE, 0, 'F', GDK_CONTROL_MASK, NULL },
     { GNOME_APP_UI_ITEM, N_("_Copy Current to Unused Pattern"), NULL, modinfo_copy_to_unused_pattern, (gpointer)0, NULL,
       GNOME_APP_PIXMAP_NONE, 0, 'G', GDK_CONTROL_MASK, NULL },
+
+    GNOMEUIINFO_SEPARATOR,
+
     { GNOME_APP_UI_ITEM, N_("C_lear Unused Patterns"), NULL, modinfo_clear_unused_patterns, (gpointer)0, NULL,
       GNOME_APP_PIXMAP_NONE, 0, 0, 0, NULL },
     { GNOME_APP_UI_ITEM, N_("_Pack Patterns"), NULL, modinfo_pack_patterns, (gpointer)0, NULL,
@@ -432,7 +531,7 @@ static GnomeUIInfo instrument_menu[] = {
 };
 
 static GnomeUIInfo settings_tracker_menu[] = {
-    { GNOME_APP_UI_TOGGLEITEM, N_("Use _Backing Store"), 0, menubar_backing_store_toggled, 0, 0,
+    { GNOME_APP_UI_TOGGLEITEM, N_("_Flicker-free scrolling"), 0, menubar_backing_store_toggled, 0, 0,
       GNOME_APP_PIXMAP_NONE, 0, 0, 0, NULL },
 
     { GNOME_APP_UI_ITEM, N_("_Previous font"), NULL, menubar_settings_tracker_prev_font, (gpointer)0, NULL,
@@ -481,7 +580,7 @@ static GnomeUIInfo help_menu[] = {
 
     GNOMEUIINFO_SEPARATOR,
 
-    { GNOME_APP_UI_ITEM, N_("Show _Tips..."), NULL, tips_dialog_create, NULL, NULL,
+    { GNOME_APP_UI_ITEM, N_("Show _Tips..."), NULL, tips_dialog_open, NULL, NULL,
       GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BOOK_RED, 0, 0, NULL },
     { GNOME_APP_UI_ITEM, N_("_XM Effects..."), NULL, cheat_sheet_dialog, NULL, NULL,
       GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BOOK_RED, 'H', GDK_CONTROL_MASK, NULL },
@@ -500,16 +599,19 @@ static GnomeUIInfo main_menu[] = {
     GNOMEUIINFO_END
 };
 
-
 void
-menubar_create (GtkWidget *window, GtkWidget *destbox)
+menubar_init_prefs ()
 {
-    gnome_app_create_menus(GNOME_APP(window), main_menu);	
+#ifndef GNOME
+    gtk_item_factory_parse_rc(prefs_get_filename("non-gnome-accels"));
+#endif
 
+    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(settings_menu[0].widget), gui_settings.gui_display_scopes);
+    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(settings_tracker_menu[0].widget), gui_settings.gui_use_backing_store);
+    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(settings_menu[9].widget), gui_settings.save_settings_on_exit);
 #ifdef NO_AUDIOFILE
     gtk_widget_set_sensitive(file_menu[SAVE_MOD_AS_WAV_POSITION].widget, FALSE);
 #endif
-
 #if !defined(DRIVER_ALSA_050)
     gtk_widget_set_sensitive(settings_menu[SETTINGS_MENU_MIDI_INDEX].widget,
 			     FALSE);
@@ -517,11 +619,19 @@ menubar_create (GtkWidget *window, GtkWidget *destbox)
 }
 
 void
-menubar_init_prefs ()
+menubar_block_mode_set(gboolean state)
 {
-  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(settings_menu[0].widget), gui_settings.gui_display_scopes);
-  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(settings_tracker_menu[0].widget), gui_settings.gui_use_backing_store);
-  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(settings_menu[9].widget), gui_settings.save_settings_on_exit);
+    mark_mode_toggle_ignore = TRUE;
+    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(edit_selection_menu[0].widget), state);
+    mark_mode_toggle_ignore = FALSE;
+}
+
+#ifdef USE_GNOME
+
+void
+menubar_create (GtkWidget *window, GtkWidget *destbox)
+{
+    gnome_app_create_menus(GNOME_APP(window), main_menu);	
 }
 
 void
@@ -532,82 +642,19 @@ menubar_write_accels ()
 
 #else
 
-static GtkItemFactoryEntry menu_items[] = {
-    { N_("/_File"),                                     NULL,          NULL, 0, "<Branch>" },
-    { N_("/File/_Open..."),                             "<control>O",  fileops_open_dialog, DIALOG_LOAD_MOD },
-    { N_("/File/Save _as..."),                          "<control>A",  fileops_open_dialog, DIALOG_SAVE_MOD },
-    { N_("/File/-"),                                    NULL,          NULL, 0, "<Separator>" },
-    { N_("/File/Save Module as _WAV..."),               NULL,          fileops_open_dialog, DIALOG_SAVE_MOD_AS_WAV },
-    { N_("/File/Save XM without samples..."),           NULL,          fileops_open_dialog, DIALOG_SAVE_SONG_AS_XM },
-    { N_("/File/-"),                                    NULL,          NULL, 0, "<Separator>" },
-    { N_("/File/_Quit"),                                "<control>Q",  menubar_quit_requested, 0 },
-    { N_("/_Module"),                                   NULL,          NULL, 0, "<Branch>" },
-    { N_("/Module/Clear _All"),                         NULL,          menubar_clear_clicked, 0 },
-    { N_("/Module/Clear _Patterns Only"),               NULL,          menubar_clear_clicked, 1 },
-    { N_("/Module/_Optimize Module"),                   NULL,          modinfo_optimize_module, 0 },
-    { N_("/_Edit"),                                     NULL,          NULL, 0, "<Branch>" },
-    { N_("/Edit/_Jazz Edit Mode"),                      "<shift>space",track_editor_toggle_jazz_edit, 0 },
-    { N_("/Edit/-"),                                    NULL,          NULL, 0, "<Separator>" },
-    { N_("/Edit/_Transposition..."),                    "<control>T",  transposition_dialog, 0 },
-    { N_("/Edit/-"),                                    NULL,          NULL, 0, "<Separator>" },
-    { N_("/Edit/_Pattern"),                             NULL,          NULL, 0, "<Branch>" },
-    { N_("/Edit/Pattern/C_ut"),                         "<alt>F3",     menubar_handle_edit_menu, 0 },
-    { N_("/Edit/Pattern/_Copy"),                        "<alt>F4",     menubar_handle_edit_menu, 1 },
-    { N_("/Edit/Pattern/_Paste"),                       "<alt>F5",     menubar_handle_edit_menu, 2 },
-    { N_("/Edit/_Track"),                               NULL,          NULL, 0, "<Branch>" },
-    { N_("/Edit/Track/C_ut"),                           "<shift>F3",   menubar_handle_edit_menu, 3 },
-    { N_("/Edit/Track/_Copy"),                          "<shift>F4",   menubar_handle_edit_menu, 4 },
-    { N_("/Edit/Track/_Paste"),                         "<shift>F5",   menubar_handle_edit_menu, 5 },
-    { N_("/Edit/Track/_Kill notes"),                    "<control>K",  menubar_handle_edit_menu, 13 },
-    { N_("/Edit/Track/_Insert"),                        NULL,          menubar_handle_edit_menu, 6 },
-    { N_("/Edit/Track/_Delete"),                        NULL,          menubar_handle_edit_menu, 7 },
-    { N_("/Edit/_Selection"),                           NULL,          NULL, 0, "<Branch>" },
-    { N_("/Edit/Selection/_Mark mode"),                 "<control>B",  menubar_mark_mode_toggled, 0, "<ToggleItem>" },
-    { N_("/Edit/Selection/C_lear block marks"),         "<control><shift>B",  menubar_handle_edit_menu, 9 },
-    { N_("/Edit/Selection/C_ut"),                       "<control>X",  menubar_handle_edit_menu, 10 },
-    { N_("/Edit/Selection/_Copy"),                      "<control>C",  menubar_handle_edit_menu, 11 },
-    { N_("/Edit/Selection/_Paste"),                     "<control>V",  menubar_handle_edit_menu, 12 },
-    { N_("/Edit/Selection/_Interpolate effects"),       "<control>I",  menubar_handle_edit_menu, 8 },
-    { N_("/Edit/Track/Increment cmd value"),            "<control>=",  menubar_handle_edit_menu, 14 },
-    { N_("/Edit/Track/Decrement cmd value"),            "<control>-",  menubar_handle_edit_menu, 15 },
-    { N_("/_Pattern"),                                  NULL,          NULL, 0, "<Branch>" },
-    { N_("/Pattern/_Find Unused Pattern"),              "<control>F",  modinfo_find_unused_pattern, 0 },
-    { N_("/Pattern/_Copy Current to Unused Pattern"),   "<control>G",  modinfo_copy_to_unused_pattern, 0 },
-    { N_("/Pattern/C_lear Unused Patterns"),            NULL,          modinfo_clear_unused_patterns, 0 },
-    { N_("/Pattern/_Pack Patterns"),                    NULL,          modinfo_pack_patterns, 0 },
-    { N_("/_Instrument"),                               NULL,          NULL, 0, "<Branch>" },
-    { N_("/Instrument/_Load XI..."),                    NULL,          fileops_open_dialog, DIALOG_LOAD_INSTRUMENT },
-    { N_("/Instrument/_Save XI..."),                    NULL,          fileops_open_dialog, DIALOG_SAVE_INSTRUMENT },
-    { N_("/Instrument/-"),                              NULL,          NULL, 0, "<Separator>" },
-    { N_("/Instrument/_Clear Current"),                 NULL,          instrument_editor_clear_current_instrument, 0 },
-    { N_("/Instrument/-"),                              NULL,          NULL, 0, "<Separator>" },
-    { N_("/Instrument/_Delete Unused Instruments"),     NULL,          modinfo_delete_unused_instruments, 0 },
-    { N_("/_Settings"),                                 NULL,          NULL, 0, "<Branch>" },
-    { N_("/Settings/Display _Oscilloscopes"),           NULL,          menubar_scopes_toggled, 0, "<ToggleItem>" },
-    { N_("/Settings/_Tracker"),                         NULL,          NULL, 0, "<Branch>" },
-    { N_("/Settings/Tracker/Use _Backing Store"),       NULL,          menubar_backing_store_toggled, 0, "<ToggleItem>" },
-    { N_("/Settings/Tracker/_Previous font"),           "<control>KP_Subtract", menubar_settings_tracker_prev_font },
-    { N_("/Settings/Tracker/_Next font"),               "<control>KP_Add", menubar_settings_tracker_next_font },
-    { N_("/Settings/-"),                                NULL,          NULL, 0, "<Separator>" },
-    { N_("/Settings/_Keyboard Configuration..."),       NULL,          keys_dialog, 0 },
-    { N_("/Settings/_Audio Configuration..."),          NULL,          audioconfig_dialog, 0 },
-    { N_("/Settings/_GUI Configuration..."),            NULL,          gui_settings_dialog, 0 },
-    { N_("/Settings/_MIDI Configuration..."),           NULL,          midi_settings_dialog, 0 },
-    { N_("/Settings/-"),                                NULL,          NULL, 0, "<Separator>" },
-    { N_("/Settings/_Save Settings now"),               NULL,          menubar_save_settings_now, 0 },
-    { N_("/Settings/Save Settings on _Exit"),           NULL,          menubar_save_settings_on_exit_toggled, 0, "<ToggleItem>" },
-    { N_("/_Help"),                                     NULL,          NULL, 0, "<Branch>" },
-    { N_("/Help/_About..."),                            NULL,          about_dialog, 0 },
-    { N_("/Help/-"),                                    NULL,          NULL, 0, "<Separator>" },
-    { N_("/Help/Show _Tips..."),                        NULL,          tips_dialog_create, 0 },
-    { N_("/Help/_XM Effects..."),                       "<control>H",  cheat_sheet_dialog, 0 },
-};
+/*
 
-static char *
-id (const char *p)
-{
-    return (char*)p;
-}
+  Coming up is the code that generates the non-GNOME menu bar from the
+  GNOME menu structure.
+
+  Earlier versions of SoundTracker used an extra array of
+  GtkItemFactoryEntry's -- problem: the translators had to do double
+  work, and if some of the entries were not translated (in a new
+  release, for example), the whole menu bar translation had to be
+  disabled, because non-translated entries could interfere with
+  translated ones.
+
+ */
 
 static void
 strip_underscores (char *dest, const char *src)
@@ -623,41 +670,105 @@ strip_underscores (char *dest, const char *src)
     *dest = '\0';
 }
 
-void
-menubar_init_prefs ()
+/* This function recursively builds an array of GtkItemFactoryEntry's
+   from the GNOME menu structure. The GtkItemFactoryEntry array is in
+   the global variable 'menubar_gtk_items'. */
+
+static int
+menubar_gnome_to_gtk (GnomeUIInfo *gnometree,
+		      int n,
+		      gchar *nameprefix)
 {
-    GtkWidget *w;
-    char buf[256];
-#ifdef ENABLE_NLS
-    char * (*tl)(const char *) = all_translated ? gettext : id;
-#else
-    char * (*tl)(const char *) = id;
-#endif
+    gchar buf[256], acc[100];
+    char *xname;
 
-    gtk_item_factory_parse_rc(prefs_get_filename("non-gnome-accels"));
+    for(; gnometree->type != GNOME_APP_UI_END; gnometree++, n++) {
+	menubar_gtk_items = g_renew(GtkItemFactoryEntry, menubar_gtk_items, n + 1);
 
-    /* UGH: Oh, oh... gtk+ requires that gtk_item_factory_get_widget()
-       be called without the underscores in the menu string */
-    strip_underscores(buf, tl("/Settings/Display _Oscilloscopes"));
-    w = gtk_item_factory_get_widget(item_factory, buf);
-    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(w), gui_settings.gui_display_scopes);
+	strcpy(buf, nameprefix);
+	strcat(buf, "/");
 
-    strip_underscores(buf, tl("/Settings/Tracker/Use _Backing Store"));
-    w = gtk_item_factory_get_widget(item_factory, buf);
-    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(w), gui_settings.gui_use_backing_store);
+	acc[0] = 0;
+	if(gnometree->shortcutmod & GDK_MOD1_MASK) {
+	    strcat(acc, "<alt>");
+	}
+	if(gnometree->shortcutmod & GDK_SHIFT_MASK) {
+	    strcat(acc, "<shift>");
+	}
+	if(gnometree->shortcutmod & GDK_CONTROL_MASK) {
+	    strcat(acc, "<control>");
+	}
+	if((xname = XKeysymToString(gnometree->shortcut))) {
+	    strcat(acc, xname);
+	}
+	menubar_gtk_items[n].accelerator = g_strdup(acc);
 
-    strip_underscores(buf, tl("/Settings/Save Settings on _Exit"));
-    w = gtk_item_factory_get_widget(item_factory, buf);
-    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(w), gui_settings.save_settings_on_exit);
+	strcat(buf, _(gnometree->title));
+	menubar_gtk_items[n].path = g_strdup(buf);
+
+	switch(gnometree->type) {
+	case GNOME_APP_UI_SUBTREE:
+	    menubar_gtk_items[n].callback = (GtkItemFactoryCallback)gnometree->func;
+	    menubar_gtk_items[n].callback_action = (guint)gnometree->funcparam;
+	    menubar_gtk_items[n].item_type = "<Branch>";
+	    strip_underscores(buf, buf);
+	    n = menubar_gnome_to_gtk((GnomeUIInfo*)gnometree->dummy1, n + 1, buf) - 1;
+	    break;
+	case GNOME_APP_UI_ITEM:
+	    menubar_gtk_items[n].callback = (GtkItemFactoryCallback)gnometree->func;
+	    menubar_gtk_items[n].callback_action = (guint)gnometree->funcparam;
+	    menubar_gtk_items[n].item_type = NULL;
+	    break;
+	case GNOME_APP_UI_TOGGLEITEM:
+	    menubar_gtk_items[n].callback = (GtkItemFactoryCallback)gnometree->func;
+	    menubar_gtk_items[n].callback_action = (guint)gnometree->funcparam;
+	    menubar_gtk_items[n].item_type = "<ToggleItem>";
+	    break;
+	case GNOME_APP_UI_SEPARATOR:
+	    menubar_gtk_items[n].callback = NULL;
+	    menubar_gtk_items[n].callback_action = 0;
+	    menubar_gtk_items[n].item_type = "<Separator>";
+	    break;
+	default:
+	    break;
+	}
+    }
+
+    return n;
+}
+
+/* After generating the GTK+ menu, this function gets GtkWidget
+   pointers to all the menu items and stores them in the GNOME
+   array. */
+
+static int
+menubar_gnome_to_gtk_get_widgets (GnomeUIInfo *gnometree,
+				  int n)
+{
+    for(; gnometree->type != GNOME_APP_UI_END; gnometree++, n++) {
+	switch(gnometree->type) {
+	case GNOME_APP_UI_SUBTREE:
+	    n = menubar_gnome_to_gtk_get_widgets((GnomeUIInfo*)gnometree->dummy1, n+1) - 1;
+	    break;
+	case GNOME_APP_UI_ITEM:
+	case GNOME_APP_UI_TOGGLEITEM:
+	    strip_underscores(menubar_gtk_items[n].path, menubar_gtk_items[n].path);
+	    gnometree->widget = gtk_item_factory_get_widget(item_factory, menubar_gtk_items[n].path);
+	    g_free(menubar_gtk_items[n].path);
+	    break;
+	default:
+	    break;
+	}
+    }
+
+    return n;
 }
 
 void
 menubar_create (GtkWidget *window, GtkWidget *destbox)
 {
     GtkWidget *thing;
-#ifdef ENABLE_NLS
-    int i;
-#endif
+    int n;
 
     accel_group = gtk_accel_group_new ();
 
@@ -665,42 +776,12 @@ menubar_create (GtkWidget *window, GtkWidget *destbox)
 					 "<main>",
 					 accel_group);
 
-    /*
+    n = menubar_gnome_to_gtk(main_menu, 0, "");
 
-      We need to check whether all menu entries have been
-      translated. If this is not the case, there will be a "File" menu
-      and a menu with a title corresponding to the translation of
-      "File", for example, if only parts of the "File" menu have been
-      translated. Since this looks rather messy, we disable
-      translation of the menu bar completely.
+    gtk_item_factory_create_items (item_factory, n, menubar_gtk_items, NULL);
 
-     */
-
-#ifdef ENABLE_NLS
-    for(i = 0, all_translated = TRUE; all_translated && i < sizeof(menu_items) / sizeof(menu_items[0]); i++) {
-        /* as long as all strings have been translated, keep all_translated = TRUE */
-        char *str = strchr(menu_items[i].path + 1, '/');
-        if(str != NULL && strlen(str) > 2) {
-            gboolean string_translated = (0 != strcmp(menu_items[i].path, _(menu_items[i].path)));
-            all_translated &= string_translated;
-            if(!string_translated) {
-                fprintf(stderr, "*** Untranslated menu entry: %s\n", menu_items[i].path);
-            }
-        }
-    }
-
-    if(all_translated) {
-        for(i = 0; i < sizeof(menu_items) / sizeof(menu_items[0]); i++) {
-            menu_items[i].path = _(menu_items[i].path);
-        }
-    } else {
-        fprintf(stderr, "*** SoundTracker Non-GNOME: Disabling menu bar translation because\n*** not all strings have been translated.\n*** ( --- this is NOT the user's fault! :D --- )\n");
-    }
-#endif
-
-    gtk_item_factory_create_items (item_factory,
-				   sizeof(menu_items) / sizeof(menu_items[0]),
-				   menu_items, NULL);
+    menubar_gnome_to_gtk_get_widgets(main_menu, 0);
+    g_free(menubar_gtk_items);
 
     gtk_accel_group_attach (accel_group, GTK_OBJECT (window));
 
@@ -717,30 +798,3 @@ menubar_write_accels ()
 
 #endif
 
-void
-menubar_block_mode_set(gboolean state)
-{
-#ifdef USE_GNOME
-
-    mark_mode_toggle_ignore = TRUE;
-    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(edit_selection_menu[0].widget), state);
-
-#else /* USE_GNOME */	
-    GtkWidget *w;
-    char buf[256];
-#ifdef ENABLE_NLS
-    char * (*tl)(const char *) = all_translated ? gettext : id;
-#else
-    char * (*tl)(const char *) = id;
-#endif
-
-    mark_mode_toggle_ignore = TRUE;
-
-    strip_underscores(buf, tl("/Edit/Selection/_Mark mode"));
-    w = gtk_item_factory_get_widget(item_factory, buf);
-    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(w), state);
-    
-#endif /* USE_GNOME */
-
-    mark_mode_toggle_ignore = FALSE;
-}
