@@ -89,7 +89,7 @@ midi_prefs midi_settings = {
 
 enum MidiSettingsPage {
   MIDI_SETTINGS_INPUT_PAGE = 0,
-  MIDI_SETTINGS_OUTPUT_PAGE, /* future plan... */
+//  MIDI_SETTINGS_OUTPUT_PAGE, /* future plan... */
   MIDI_SETTINGS_MISC_PAGE,
   MIDI_SETTINGS_COUNT_OF_PAGES
 };
@@ -320,6 +320,7 @@ dialog_apply_callback (GtkWidget *notebook)
 
     break;
 
+#if 0
   case MIDI_SETTINGS_OUTPUT_PAGE:
     if (IS_MIDI_DEBUG_ON) {
       g_print( "new output settings: client %d port %d\n",
@@ -328,6 +329,7 @@ dialog_apply_callback (GtkWidget *notebook)
     }
 
     break;
+#endif
 
   case MIDI_SETTINGS_MISC_PAGE:
 
@@ -503,6 +505,7 @@ create_midi_notebook( GtkDialog* dialog, midi_prefs settings)
   gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                            page, gtk_label_new(_("Input")));
 
+#if 0 /* mkrause 20020516 */
   /*****************************************************************/
   /* Create the Output page */
 
@@ -516,6 +519,7 @@ create_midi_notebook( GtkDialog* dialog, midi_prefs settings)
 
   gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                            page, gtk_label_new(_("Output")));
+#endif
 
   /*****************************************************************/
   /* Create the Misc page */
