@@ -361,11 +361,40 @@ modinfo_clear_unused_patterns (void)
 }
 
 // Optimize -- clear everything unused
+
+static void
+modinfo_optimize_module_callback (gint reply,
+		   gpointer data)
+{
+    if(reply == 0) {
+	modinfo_clear_unused_patterns();
+    	modinfo_delete_unused_instruments();
+	modinfo_reorder_patterns();
+    	xm_set_modified(1);
+    }
+}
+
 void
 modinfo_optimize_module (void)
 {
-    modinfo_clear_unused_patterns();
-    modinfo_delete_unused_instruments();
-    modinfo_reorder_patterns();
-    xm_set_modified(1);
+    char infbuf[512];
+    int a, b, c, d, e;
+
+    d = sizeof(xm->patterns) / sizeof(xm->patterns[0]);
+    e = sizeof(xm->instruments) / sizeof(xm->instruments[0]);
+    for(a = 0, b = 0, c = 0; a < d; a++) {
+	if(!st_is_pattern_used_in_song(xm, a))
+	    b++;
+	
+        if(a<e)
+	    if(!st_instrument_used_in_song(xm, a + 1)) c++;
+    }
+
+    sprintf(infbuf, _("Unused patterns: %d (used: %d)\nUnused instruments: %d (used: %d)\n\nClear unused and reorder playlist?\n"),
+					b, d-b, c, e-c);
+
+    gnome_app_ok_cancel_modal(GNOME_APP(mainwindow),
+			      infbuf, modinfo_optimize_module_callback, NULL);
+
 }
+
