@@ -147,6 +147,10 @@ playlist_songpos_changed (GtkSpinButton *spin,
 
     g_assert(IS_PLAYLIST(p));
 
+    /* In gtk+-1.2.10, the GtkAdjustment loses this setting when the
+       upper value is changed by the program. */
+    spin->adjustment->page_increment = 5.0;
+
     newpos = gtk_spin_button_get_value_as_int(spin);
 
     if(newpos == p->current_position)
@@ -224,6 +228,10 @@ playlist_restartpos_changed (GtkSpinButton *spin,
 
     g_assert(IS_PLAYLIST(p));
 
+    /* In gtk+-1.2.10, the GtkAdjustment loses this setting when the
+       upper value is changed by the program. */
+    spin->adjustment->page_increment = 5.0;
+
     if(p->restart_position == n)
 	return;
 
@@ -276,7 +284,6 @@ playlist_delete_clicked (GtkWidget *w,
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(p->spin_songpat), p->patterns[pos]);
 }
 
-
 GtkWidget *
 playlist_new (void)
 {
