@@ -20,6 +20,7 @@
  */
 
 #include <stdlib.h>
+#include <string.h>
 
 #include "sample-display.h"
 
@@ -347,10 +348,10 @@ sample_display_draw_data (GdkDrawable *win,
     gc = color ? s->bg_gc : s->fg_gc;
 
     if(s->datatype == 16) {
-	c = ((gint16*)s->data)[OFFSET_RANGE(s->datalen, XPOS_TO_OFFSET(x - 1))];
+	c = -((gint16*)s->data)[OFFSET_RANGE(s->datalen, XPOS_TO_OFFSET(x - 1))];
 	
 	while(width >= 0) {
-	    d = ((gint16*)s->data)[OFFSET_RANGE(s->datalen, XPOS_TO_OFFSET(x))];
+	    d = -((gint16*)s->data)[OFFSET_RANGE(s->datalen, XPOS_TO_OFFSET(x))];
 	    gdk_draw_line(win, gc,
 			  x - 1, ((c + 32768) * sh) >> 16,
 			  x,     ((d + 32768) * sh) >> 16);
@@ -359,10 +360,10 @@ sample_display_draw_data (GdkDrawable *win,
 	    width--;
 	}
     } else {
-	c = ((gint8*)s->data)[OFFSET_RANGE(s->datalen, XPOS_TO_OFFSET(x - 1))];
+	c = -((gint8*)s->data)[OFFSET_RANGE(s->datalen, XPOS_TO_OFFSET(x - 1))];
 	
 	while(width >= 0) {
-	    d = ((gint8*)s->data)[OFFSET_RANGE(s->datalen, XPOS_TO_OFFSET(x))];
+	    d = -((gint8*)s->data)[OFFSET_RANGE(s->datalen, XPOS_TO_OFFSET(x))];
 	    gdk_draw_line(win, gc,
 			  x - 1, ((c + 128) * sh) >> 8,
 			  x,     ((d + 128) * sh) >> 8);
