@@ -74,6 +74,9 @@ static GtkWidget *spin_loopstart, *spin_loopend;
 static struct SampleEditor { // simplifies future encapsulation of this into a Gtk+ widget
     GtkWidget *label_selection;
     GtkWidget *label_length;
+    gchar label_selection_new_text[64];
+    gchar label_length_new_text[64];
+    guint idle_handler;
     GtkWidget *vertical_boxes[3];
 } sample_editor;
 static struct SampleEditor * const se = &sample_editor;
@@ -493,17 +496,44 @@ sample_editor_blocked_set_loop_spins (int start,
     sample_editor_block_loop_spins(0);
 }
 
+static gint
+sample_editor_idle_draw_function (struct SampleEditor *se)
+{
+    gtk_label_set(GTK_LABEL(se->label_selection), se->label_selection_new_text);
+    gtk_label_set(GTK_LABEL(se->label_length), se->label_length_new_text);
+    gtk_idle_remove(se->idle_handler);
+    se->idle_handler = 0;
+    return TRUE;
+}
+
 static void
 sample_editor_set_selection_label (int start,
 				   int end)
 {
-    char buf[64];
+    STSample *sts = current_sample;
 
     if(start > -1) {
-	sprintf(buf, ("%d - %d"), start, end);
-	gtk_label_set(GTK_LABEL(se->label_selection), buf);
+	sprintf(se->label_selection_new_text, ("%d - %d"), start, end);
+	sprintf(se->label_length_new_text, ("%d"), end-start);
     } else {
-	gtk_label_set(GTK_LABEL(se->label_selection), _("(no selection)"));
+	strcpy(se->label_selection_new_text, _("(no selection)"));
+	sprintf(se->label_length_new_text, ("%d"), sts->sample.length);
+    }
+
+    /* Somewhere on the way to gtk+-1.2.10, gtk_label_set_text() has
+       become incredibly slow. So slow that my computer is overwhelmed
+       with the frequent calls to this function when one does a
+       selection with the mouse -- with the consequence that the idle
+       function updating the visual selection in the sample display
+       widget is called only very seldom.
+
+       That's why I'm doing the gtk_label_set_text() only if the
+       computer is idle. */
+
+    if(!se->idle_handler) {
+	se->idle_handler = gtk_idle_add((GtkFunction)sample_editor_idle_draw_function,
+						  (gpointer)se);
+	g_assert(se->idle_handler != 0);
     }
 }
 
@@ -1512,7 +1542,7 @@ sample_editor_open_raw_sample_dialog (const gchar *filename)
     gtk_widget_show(box2);
     gtk_box_pack_start(GTK_BOX(box1), box2, FALSE, TRUE, 0);
 
-    thing = gtk_label_new(_("Wortformat:"));
+    thing = gtk_label_new(_("Word format:"));
     gtk_widget_show(thing);
     gtk_box_pack_start(GTK_BOX(box2), thing, FALSE, TRUE, 0);
     add_empty_hbox(box2);
@@ -1630,6 +1660,7 @@ sample_editor_load_wav (void)
     return;
 }
 
+
 static void
 sample_editor_save_wav_main (const gchar *fn,
 			     guint32 offset,
@@ -1639,6 +1670,7 @@ sample_editor_save_wav_main (const gchar *fn,
     AFfilesetup outfilesetup;
     double rate = 44100.0;
 
+
     statusbar_update(STATUS_SAVING_SAMPLE, TRUE);
 
     file_selection_save_path(fn, gui_settings.savesmpl_path);
@@ -1692,10 +1724,21 @@ sample_editor_save_wav_main (const gchar *fn,
     statusbar_update(STATUS_SAMPLE_SAVED, FALSE);
 }
 
+static void
+sample_editor_save_wav_callback (gint reply,
+		   gpointer data)
+{
+    if(reply == 0) {
+		// save entire sample
+		sample_editor_save_wav_main((gchar*)data, 0, current_sample->sample.length);
+    }
+}
+
 static void
 sample_editor_save_wav (void)
 {
-    const gchar *fn;
+    gchar *fn;
+	FILE *f;
 
     fn = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileops_dialogs[DIALOG_SAVE_SAMPLE]));
 
@@ -1705,14 +1748,35 @@ sample_editor_save_wav (void)
     if(current_sample->sample.data == NULL)
 	return;
 
-    sample_editor_save_wav_main(fn, 0, current_sample->sample.length);
+	f = fopen(fn, "r");
+
+	if(f != NULL) {
+	    fclose(f);
+	    gnome_app_ok_cancel_modal(GNOME_APP(mainwindow),
+				      _("Are you sure you want to overwrite the file?"),
+				      sample_editor_save_wav_callback, fn);
+	} else {
+		sample_editor_save_wav_callback(0, fn);
+    }
+}
+
+static void
+sample_editor_save_region_callback (gint reply,
+		   gpointer data)
+{
+    int rss = sampledisplay->sel_start, rse = sampledisplay->sel_end;
+
+    if(reply == 0) {
+        // save region
+		sample_editor_save_wav_main((gchar*)data, rss, rse - rss);
+    }
 }
 
 static void
 sample_editor_save_region_wav (void)
 {
-    const gchar *fn;
-    int rss = sampledisplay->sel_start, rse = sampledisplay->sel_end;
+    gchar *fn;
+    FILE *f;
 
     fn = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileops_dialogs[DIALOG_SAVE_RGN_SAMPLE]));
 
@@ -1722,12 +1786,21 @@ sample_editor_save_region_wav (void)
     if(current_sample->sample.data == NULL)
 	return;
 
-    if(rss == -1) {
-        error_error(_("Nothing to save."));
+    if(sampledisplay->sel_start == -1) {
+        error_error(_("Please select region first."));
 	return;
     }
 
-    sample_editor_save_wav_main(fn, rss, rse - rss);
+	f = fopen(fn, "r");
+
+	if(f != NULL) {
+	    fclose(f);
+	    gnome_app_ok_cancel_modal(GNOME_APP(mainwindow),
+				      _("Are you sure you want to overwrite the file?"),
+				      sample_editor_save_region_callback, fn);
+	} else {
+	    sample_editor_save_region_callback(0, fn);
+    }
 }
 
 #endif /* NO_AUDIOFILE */
