@@ -59,6 +59,9 @@ static int note_running[96];
 static int update_freq = 30;
 static int gtktimer = -1;
 
+static guint track_editor_editmode_status_idle_handler = 0;
+static gchar track_editor_editmode_status_ed_buf[128];
+
 /* jazz edit stuff */
 static GtkWidget *jazzbox;
 static GtkWidget *jazztable;
@@ -73,12 +76,27 @@ static void update_hscrollbar(Tracker *t, int leftchan, int numchans, int dispch
 static void update_mainmenu_blockmark(Tracker *t, int state);
 static gboolean track_editor_handle_column_input(Tracker *t, int gdkkey);
 
+static gint
+track_editor_editmode_status_idle_function (void)
+{
+#ifdef USE_GNOME
+    gnome_appbar_set_status(GNOME_APPBAR(status_bar), track_editor_editmode_status_ed_buf);
+#else
+    gtk_statusbar_pop(GTK_STATUSBAR(status_bar), statusbar_context_id);
+    gtk_statusbar_push(GTK_STATUSBAR(status_bar), statusbar_context_id, track_editor_editmode_status_ed_buf);
+#endif
+
+    gtk_idle_remove(track_editor_editmode_status_idle_handler);
+    track_editor_editmode_status_idle_handler = 0;
+    return TRUE;
+}
+
 void
 show_editmode_status(void)
 {
     Tracker *t = tracker;
     XMNote *note = &t->curpattern->channels[t->cursor_ch][t->patpos];
-    gchar ed_buf[128], tmp_buf[128];
+    gchar tmp_buf[128];
     int cmd_p1, cmd_p2;
     
     static const gchar *fx_commands[]={
@@ -158,26 +176,26 @@ show_editmode_status(void)
         N_("square"),                    /* 2 */
     };
 
-    sprintf(ed_buf, "[Chnn: %02d] [Pos: %03d] [Instr: %03d] ", t->cursor_ch+1,
+    sprintf(track_editor_editmode_status_ed_buf, "[Chnn: %02d] [Pos: %03d] [Instr: %03d] ", t->cursor_ch+1,
                                                                t->patpos,
                                                                note->instrument);
 
-    strcat(ed_buf, "[Vol: ");
+    strcat(track_editor_editmode_status_ed_buf, "[Vol: ");
     
     switch(note->volume&0xf0)
     {
     case 0:
-        strcat(ed_buf, _("None"));
+        strcat(track_editor_editmode_status_ed_buf, _("None"));
         break;
     case 0x10: case 0x20: case 0x30: case 0x40: case 0x50:
-        strcat(ed_buf, _("Set volume"));
+        strcat(track_editor_editmode_status_ed_buf, _("Set volume"));
         break;
     default:
         if(note->volume>0xf) {
             sprintf(tmp_buf,"%s", vol_fx_commands[((note->volume&0xf0)-0x60)>>4]);
-            strcat(ed_buf, tmp_buf);
+            strcat(track_editor_editmode_status_ed_buf, tmp_buf);
         } else
-            strcat(ed_buf, _("None"));
+            strcat(track_editor_editmode_status_ed_buf, _("None"));
         break;
     }
 
@@ -188,10 +206,10 @@ show_editmode_status(void)
             sprintf(tmp_buf, " => %02d ] ", note->volume&0xf);
     } else
          sprintf(tmp_buf, " ] ");
-    strcat(ed_buf, tmp_buf);
+    strcat(track_editor_editmode_status_ed_buf, tmp_buf);
         
     memset(tmp_buf, 0, strlen(tmp_buf));
-    strcat(ed_buf, "[Cmd: ");
+    strcat(track_editor_editmode_status_ed_buf, "[Cmd: ");
 
     /* enum values maybe for fx numbers ? */
     switch(note->fxtype)
@@ -236,7 +254,7 @@ show_editmode_status(void)
         sprintf(tmp_buf, "%s", fx_commands[note->fxtype]);
         break;
     }
-    strcat(ed_buf, tmp_buf);
+    strcat(track_editor_editmode_status_ed_buf, tmp_buf);
 
     memset(tmp_buf, 0, strlen(tmp_buf));
 
@@ -283,14 +301,14 @@ show_editmode_status(void)
         sprintf(tmp_buf, "]");
         break;
     }
-    strcat(ed_buf, tmp_buf);
-    
-#ifdef USE_GNOME
-    gnome_appbar_set_status(GNOME_APPBAR(status_bar), ed_buf);
-#else
-    gtk_statusbar_pop(GTK_STATUSBAR(status_bar), statusbar_context_id);
-    gtk_statusbar_push(GTK_STATUSBAR(status_bar), statusbar_context_id, ed_buf);
-#endif
+    strcat(track_editor_editmode_status_ed_buf, tmp_buf);
+
+    if(!track_editor_editmode_status_idle_handler) {
+	track_editor_editmode_status_idle_handler = gtk_idle_add(
+	    (GtkFunction)track_editor_editmode_status_idle_function,
+	    NULL);
+	g_assert(track_editor_editmode_status_idle_handler != 0);
+    }
 }
 
 void
@@ -968,12 +986,14 @@ void
 track_editor_copy_selection (Tracker *t)
 {
     track_editor_copy_cut_selection_common(t, FALSE);
+    menubar_block_mode_set(FALSE);
 }
 
 void
 track_editor_cut_selection (Tracker *t)
 {
     track_editor_copy_cut_selection_common(t, TRUE);
+    menubar_block_mode_set(FALSE);
     xm->modified = 1;
     tracker_redraw(t);
 }
