@@ -590,7 +590,7 @@ print_channel_numbers (GtkWidget *widget,
     char buf[4];
     Tracker *t = TRACKER(widget);
 
-    if(!gui_settings.channel_numbering) {
+    if(0 /* !gui_settings.channel_numbering */) {
 	return;
     }
 
@@ -706,7 +706,7 @@ tracker_draw_clever (GtkWidget *widget,
 	    y += (t->disp_rows - absdist) * fonth;
 	} else if(dist < 0) {
 	    /* go up in pattern -- scroll down */
-	    if(gui_settings.channel_numbering) {
+	    if(1 /* gui_settings.channel_numbering */) {
 		/* Redraw line displaying the channel numbers before scrolling down */
 		print_notes_and_bars(widget, win,
 				     0, t->disp_starty,
@@ -947,14 +947,21 @@ tracker_set_font (Tracker *t,
 		  const gchar *fontname)
 {
     GdkFont *font;
+    int fonth, fontw;
 
     if((font = gdk_font_load(fontname))) {
-	gdk_font_unref(t->font);
-	t->font = font;
-	t->fonth = t->font->ascent + t->font->descent + t->baselineskip;
-	t->fontw = gdk_string_width(t->font, "X"); /* let's just hope this is a non-proportional font */
-	tracker_reset(t);
-	return TRUE;
+	fonth = font->ascent + font->descent + t->baselineskip;
+	fontw = gdk_string_width(font, "X"); /* let's just hope this is a non-proportional font */
+
+	/* Some fonts have width 0, for example 'clearlyu' */
+	if(fonth >= 1 && fontw >= 1) {
+	    gdk_font_unref(t->font);
+	    t->font = font;
+	    t->fontw = fontw;
+	    t->fonth = fonth;
+	    tracker_reset(t);
+	    return TRUE;
+	}
     }
 
     return FALSE;
