@@ -78,6 +78,21 @@ transposition_for_each (void (*function)(XMNote *track, int patlen, int data),
     }
 }
 
+static void
+transposition_transpose_notes_full (XMNote *track,
+				    int length,
+				    int add,
+				    int instrument)
+{
+    while(length--) {
+	if(track->note != 0 && track->note != 97
+	   && (instrument == -1 || instrument == track->instrument)) {
+	    track->note = CLAMP(track->note + add, 1, 96);
+	}
+	track++;
+    }
+}
+
 static void
 transposition_transpose_notes_sub (XMNote *track,
 				   int patlen,
@@ -86,13 +101,7 @@ transposition_transpose_notes_sub (XMNote *track,
     gboolean all_instruments = find_current_toggle(transposition_instrument_mode_w, 2);
     int current_instrument = gui_get_current_instrument();
 
-    while(patlen--) {
-	if(track->note != 0 && track->note != 97
-	   && (all_instruments || current_instrument == track->instrument)) {
-	    track->note = CLAMP(track->note + add, 1, 96);
-	}
-	track++;
-    }
+    transposition_transpose_notes_full(track, patlen, add, all_instruments ? current_instrument : -1);
 }
 
 static void
@@ -138,6 +147,28 @@ transposition_transpose_notes (GtkWidget *w,
     xm_set_modified(1);
 }
 
+void
+transposition_transpose_selection (Tracker *t,
+				   int by)
+{
+    int height, width, chStart, rowStart;
+    int i;
+
+    if(!tracker_is_valid_selection(t)) {
+	return;
+    }
+
+    tracker_get_selection_rect(t, &chStart, &rowStart, &width, &height);
+
+    for(i = chStart; i < chStart + width; i++) {
+	transposition_transpose_notes_full(t->curpattern->channels[i] + rowStart,
+					   height, by, -1);
+    }
+
+    xm_set_modified(1);
+    tracker_redraw(t);
+}
+
 static void
 transposition_change_instruments (GtkWidget *w,
 				  gpointer data)
