@@ -33,6 +33,7 @@
 #include <sys/stat.h>
 #include <math.h>
 #include <sys/wait.h>
+#include <time.h>
 
 #include <glib.h>
 
@@ -46,6 +47,8 @@
 #include "errors.h"
 #include "audio.h"
 
+#include "preferences.h"
+
 #define LFSTAT_IS_MODULE 1
 
 static guint16 npertab[60]={
@@ -429,6 +432,8 @@ xm_check_envelope (STEnvelope *e)
 	if(!(h >= 0 && h <= 64))
 	    e->points[i].val = 32;
     }
+
+    e->points[0].pos = 0;
 }
 
 static int
@@ -919,7 +924,7 @@ XM_Load (const char *filename,int *status)
     }
 
     if(get_le_32(xh + 60) != 276) {
-	error_warning(_("XM header length != 276. Maybe a pre-0.0.12 SoundTracker module? :-)\n"));
+	error_warning("XM header length != 276. Maybe a pre-0.0.12 SoundTracker module? :-)\n");
     }
 
     if(get_le_16(xh + 58) != 0x0104) {
@@ -1121,6 +1126,32 @@ XM_Free (XM *xm)
 **************************************************************************/
 char *err_msg = "Bzzzz, error extracting song, aborting operation.";
 
+
+static char *st_tmpnam (char *tmp_path)
+{
+static char tname[1024], tfname[32];
+char *tmpchr = "abcdefghijklmnopqrstuvwxyz1234567890ABCDEFGHIJKLMNOPQRSTUVWXYZ";
+int i;
+
+    srand (time(NULL));
+
+    strcpy (tname, prefs_get_prefsdir());
+    strcat (tname, "/tmp/");
+    sprintf (tfname, "st%d", getpid());
+    strcat (tname, tfname);
+
+    for(i=0; i < 16; i++)
+        tfname[i] = tmpchr[(int) ((62.0 * rand()) / RAND_MAX)];
+    tfname[i]='\0';
+
+    strcat (tname, tfname);
+
+	if (tmp_path != NULL)
+		strcpy (tmp_path, tname);
+
+	return tname;
+}
+
 static char *escape_filename(const char *filename)
 {
     GString *escaped = g_string_new("");
@@ -1307,7 +1338,7 @@ File_Load (const char *filename)
     g_assert(filename != NULL);
 
     filename_esc = escape_filename(filename);
-    tmpnam(tmp_path);
+    st_tmpnam(tmp_path);
 
     /* test and load zip files. for unzip version 5.31 */
     if (f_extension_cmp (filename,".zip")) {
